@@ -1,20 +1,3 @@
-/*
-   This file is part of SKS.  SKS is free software; you can
-   redistribute it and/or modify it under the terms of the GNU General
-   Public License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-   USA 
-*/
-
 // Stubs appropriate for Berkeley DB 4.x
 #include <stdio.h>
 #include <stdlib.h>
@@ -240,11 +223,6 @@ void raise_db_cb(const char *prefix, char *msg) { raise_db(msg); }
 
 // Declaration of flag enums in ocaml must be in same order as in C
 
-static int dbenv_create_flags[] = { DB_CLIENT };
-
-//+
-//+   type create_flag = CLIENT
-
 static int dbenv_open_flags[] = {
   DB_JOINENV, DB_INIT_CDB, DB_INIT_LOCK, DB_INIT_LOG, DB_INIT_MPOOL, 
   DB_INIT_TXN, DB_RECOVER, DB_RECOVER_FATAL, DB_USE_ENVIRON,
@@ -270,12 +248,12 @@ static int dbenv_verbose_flags[] = {
 /**  DBENV Calls  *******************************************/
 //+
 
-//+   external create : create_flag list -> t = "caml_dbenv_create"
-value caml_dbenv_create(value vflags){
-  CAMLparam1(vflags);
+//+   external create : unit -> t = "caml_dbenv_create"
+value caml_dbenv_create(value unit){
+  CAMLparam1(unit);
   CAMLlocal1(rval);
   int err;
-  int flags = convert_flag_list(vflags,dbenv_create_flags);
+  int flags = 0;
   DB_ENV *dbenv;
   
   err = db_env_create(&dbenv,flags);
@@ -313,7 +291,7 @@ value caml_dbenv_open(value dbenv, value vdirectory,
 }
 // simple open, combination of create and open
 //+   let sopen dirname flags mode = 
-//+     let dbenv = create [] in
+//+     let dbenv = create () in
 //+     dopen dbenv dirname flags mode;
 //+     dbenv
 
