@@ -25,6 +25,7 @@ module Unix = UnixLabels
 
 exception Bug of string
 exception Transaction_aborted of string
+exception Argument_error of string
 
 module Map = PMap.Map 
 let (|<) map key = (fun data -> Map.add ~key ~data map)
@@ -35,7 +36,7 @@ let (|=) map key = Map.find key map
 (** filters applied to all incoming keys *)
 let enforced_filters = ["yminsky.dedup"]
 
-let version_tuple = (1,0,5)
+let version_tuple = (__VERSION__)
 let compatible_version_tuple = (0,1,5)
 let version = 
   let (maj_version,min_version,release) = version_tuple in
@@ -61,7 +62,7 @@ let stored_logfile_name = ref None
 
 (**************************************************************************)
 
-let plerror level = 
+let plerror level format = 
   kprintf (fun s -> 
 	     if !Settings.debug && level  <= !Settings.debuglevel 
 	     then  (
@@ -73,21 +74,18 @@ let plerror level =
 	       output_string !logfile s; 
 	       output_string !logfile "\n";
 	       flush !logfile;
-	     );
-	     s
-	  )
+	     ) )
+    format
 
 (**************************************************************************)
 
 let set_logfile extension = 
-  match !Settings.logfile with
-    | None -> ()
-    | Some fname ->
-	let fname = (Filename.concat !Settings.basedir fname) ^ extension in
-	stored_logfile_name := Some fname;
-	logfile := open_out_gen [ Open_wronly; Open_creat; Open_append; ] 
-	  0o600 fname;
-	ignore (plerror 0 "Opening log")
+  if !Settings.filelog then
+    let fname = (Filename.concat !Settings.basedir extension) ^ ".log" in
+    stored_logfile_name := Some fname;
+    logfile := open_out_gen [ Open_wronly; Open_creat; Open_append; ] 
+      0o600 fname;
+    plerror 0 "Opening log"
 
 let reopen_logfile () =
   match !stored_logfile_name with
@@ -101,7 +99,7 @@ let reopen_logfile () =
 
 let perror x = plerror 3 x
 
-let eplerror level e = 
+let eplerror level e format = 
   kprintf (fun s ->
 	     if !Settings.debug && level  <= !Settings.debuglevel 
 	     then  (
@@ -113,14 +111,12 @@ let eplerror level e =
 	       output_string !logfile s;
 	       fprintf !logfile ": %s\n" (err_to_string e);
 	       flush !logfile;
-	     );
-	     s
+	     )
 	  )
+    format
 
 let eperror x = eplerror 3 x
 
-let signore (x:string) = ignore x
-
 (********************************************************************)
 (** Setup signals.  In particular, most of the time we want to catch and
   gracefully handle both sigint and sigterm *)
@@ -215,3 +211,7 @@ let recon_addr_to_http_addr addr = match addr with
   | Unix.ADDR_INET (inet_addr,port) -> Unix.ADDR_INET (inet_addr,port + 1)
 
 
+let get_client_recon_addr () =
+  Unix.ADDR_INET (Unix.inet_addr_of_string recon_address,0)
+let get_client_recon_addr = 
+  Utils.unit_memoize get_client_recon_addr
