@@ -32,18 +32,18 @@ struct
   open Pstyle
 
   let () = 
-    set_logfile ".db";
-    ignore (plerror 0 "sks_db, SKS version %s" version); 
-    ignore (plerror 0 "Copyright Yaron Minsky 2002"); 
-    ignore (plerror 0 "Licensed under GPL.  See COPYING file for details"); 
-    ignore (plerror 3 "http port: %d" http_port)
+    set_logfile "db";
+    plerror 0 "sks_db, SKS version %s" version; 
+    plerror 0 "Copyright Yaron Minsky 2002, 2003, 2004"; 
+    plerror 0 "Licensed under GPL.  See COPYING file for details"; 
+    plerror 3 "http port: %d" http_port
 
   let settings = {
     Keydb.withtxn = !Settings.transactions;
     Keydb.cache_bytes = !Settings.cache_bytes;
     Keydb.pagesize = !Settings.pagesize;
-    Keydb.dbdir = !Settings.dbdir;
-    Keydb.dumpdir = !Settings.dumpdir;
+    Keydb.dbdir = Lazy.force Settings.dbdir;
+    Keydb.dumpdir = Lazy.force Settings.dumpdir;
   }
   module Keydb = Keydb.Safe
 
@@ -51,7 +51,7 @@ struct
      code for the DB. *)
 
   let withtxn = !Settings.transactions 
-  let dbdir = !Settings.dbdir
+  let dbdir = Lazy.force Settings.dbdir
   let () = 
     if not withtxn then 
       failwith "Running sks_db without transactions is no longer supported."
@@ -65,20 +65,20 @@ struct
   let comsock = Eventloop.create_sock db_command_addr
 
 
-  (****************************************************************************)
+  (*********************************************************************)
   (** Database checkpointing and syncing *)
 
   let sync () = 
-    ignore (perror "Syncing database");
+    perror "Syncing database";
     Keydb.sync ();
-    ignore (perror "Syncing complete")
+    perror "Syncing complete"
 
   let sync_interval = !Settings.db_sync_interval
 
   let checkpoint () = 
-    ignore (perror "Checkpointing database");
+    perror "Checkpointing database";
     Keydb.checkpoint ();
-    ignore (perror "Checkpointing complete")
+    perror "Checkpointing complete"
       
   let checkpoint_interval = !Settings.checkpoint_interval
 
@@ -120,10 +120,10 @@ struct
   let last_stat_page = ref (Stats.generate_html_stats_page_nostats ())
 
   let calculate_stats_page () = 
-    signore (plerror 3 "Calculating DB stats"); 
+    plerror 3 "Calculating DB stats"; 
     let (log,size) = get_stats () in
     last_stat_page := Stats.generate_html_stats_page log size;
-    signore (plerror 3 "Done calculating DB stats"); 
+    plerror 3 "Done calculating DB stats"; 
     []
 
   let get_keys_by_keyid keyid =
@@ -148,9 +148,16 @@ struct
       | _ -> failwith "unknown keyid type"
 	  
 
+  (** returns list of keys readied for presentation *)
+  let clean_keys request keys = 
+    if request.clean 
+    then Utils.filter_map ~f:Fixkey.presentation_filter keys 
+    else keys
+
   (** return uid given keyid *)
-  let get_uids keyid = 
+  let get_uids request keyid = 
     let keys = get_keys_by_keyid keyid in
+    let keys = clean_keys request keys in
     match keys with
       | [] | _::_::_ -> []
       | key::tl ->
@@ -164,16 +171,39 @@ struct
     String.length string >= String.length prefix &&
     (String.sub ~pos:0 ~len:(String.length prefix) string = prefix)
 
-  let lookup_keys search_terms =
+  (******************************************************************)
+
+  (** Filters a set of keys down to those for which the given string appears
+      literally within the UID *)
+  let filter_keys_exact keys search = 
+    List.filter keys
+      ~f:(fun key -> 
+	    let uids = (KeyMerge.key_to_pkey key).KeyMerge.uids in
+	    List.exists uids 
+	      ~f:(fun (uid_packet,_) -> 
+		    let uid = uid_packet.packet_body in
+		    Utils.substring_find search uid <> -1)
+	 )
+    
+  (******************************************************************)
+
+  let lookup_keys request  =
+    let search_terms = request.search in
     let keys = 
-      match search_terms with
+      match request.search with
 	| [] -> []
 	| first::rest ->
 	    if check_prefix first "0x" then 
 	      (* keyid search *)
 	      let keyid_string_length = String.length first - 2 in
 	      let keyid = 
-		KeyHash.dehexify (String.sub ~pos:2 ~len:keyid_string_length first)
+		try
+		  KeyHash.dehexify 
+		    (String.sub ~pos:2 ~len:keyid_string_length first)
+		with 		    e -> 
+		  let exn_str = sprintf "Unable to parse hash string: %s"
+		    (Printexc.to_string e) in
+		  raise (Wserver.Misc_error exn_str)
 	      in
 	      let keys = (try get_keys_by_keyid keyid 
 			  with Failure s -> raise (Wserver.Misc_error s))
@@ -181,28 +211,32 @@ struct
 	      keys
 	    else 
 	      let keys = Keydb.get_by_words ~max:!Settings.max_matches 
-			   search_terms 
+			   request.search 
 	      in
 	      tsort_keys keys
     in
+    let keys = 
+      if request.exact then filter_keys_exact keys request.search_string
+      else keys
+    in
     if keys = [] then raise (Wserver.Misc_error "No keys found")
     else keys
 
-
   (******************************************************************)
 
   let handle_get_request request =
     match request.kind with
       | Stats -> 
-	  signore (plerror 4 "/pks/lookup: DB Stats request");
-	  ("text/html", !last_stat_page)
+	  plerror 4 "/pks/lookup: DB Stats request";
+	  ("text/html; charset=UTF-8", !last_stat_page)
       | Get -> 
-	  signore (plerror 4 "/pks/lookup: Get request (%s)"
-		     (String.concat " " request.search));
-	  let keys = lookup_keys request.search in
+	  plerror 4 "/pks/lookup: Get request (%s)"
+	    (String.concat " " request.search);
+	  let keys = lookup_keys request in
+	  let keys = clean_keys request keys in
 	  let keystr = Key.to_string_multiple keys in
 	  let aakeys = Armor.encode_pubkey_string keystr in
-	  ("text/html",
+	  ("text/html; charset=UTF-8",
 	   HtmlTemplates.page  
 	     ~title:(sprintf "Public Key Server -- Get ``%s ''" 
 		       (String.concat ~sep:" " request.search))
@@ -210,16 +244,23 @@ struct
 	  )
       | HGet -> 
 	  let hash_str = List.hd request.search in
-	  signore (plerror 4 "/pks/lookup: Hash search: %s" hash_str);
+	  plerror 4 "/pks/lookup: Hash search: %s" hash_str;
 	  let hash = KeyHash.dehexify hash_str in
 	  flush Pervasives.stdout;
 	  let key = 
 	    try Keydb.get_by_hash hash with Not_found -> 
 	      raise (Wserver.Misc_error "Requested hash not found")
 	  in
+	  let key = 
+	    if request.clean then
+	      match Fixkey.presentation_filter key with
+		  None -> raise (Wserver.Misc_error "No valid key found for hash")
+		| Some key -> key
+	    else key
+	  in
 	  let keystr = Key.to_string key in
 	  let aakeys = Armor.encode_pubkey_string keystr in
-	  ("text/html",
+	  ("text/html; charset=UTF-8",
 	   HtmlTemplates.page  
 	     ~title:(sprintf "Public Key Server -- Get ``%s ''" hash_str)
 	     ~body:(sprintf "\r\n<pre>\r\n%s\r\n</pre>\r\n" aakeys)
@@ -227,9 +268,11 @@ struct
 
       | Index | VIndex ->  
 	  (* VIndex requests are treated indentically to index requests *)
-	  signore (plerror 4 "/pks/lookup: Index request: (%s)" 
-		     (String.concat " " request.search));
-	  let keys = lookup_keys request.search in
+	  plerror 4 "/pks/lookup: Index request: (%s)" 
+	    (String.concat " " request.search);
+	  let keys = lookup_keys request in
+	  let hashes = List.map ~f:KeyHash.hash keys in
+	  let keys = clean_keys request keys in
 	  if request.machine_readable then 
 	    ("text/plain",
 	     MRindex.keys_to_index keys)
@@ -238,15 +281,18 @@ struct
 	      try
 		let output = 
 		  if request.kind = VIndex then
-		    List.map ~f:(Index.key_to_lines_verbose ~get_uids request) keys 
+		    List.map2 keys hashes
+		      ~f:(Index.key_to_lines_verbose 
+			    ~get_uids:(get_uids request) request) 
 		  else
-		    List.map ~f:(Index.key_to_lines_normal request) keys 
+		    List.map2 keys hashes
+		      ~f:(Index.key_to_lines_normal request) 
 		in
 		let output = List.flatten output in
 		let pre = HtmlTemplates.preformat_list 
 			    (Index.keyinfo_header request :: output)
 		in
-		("text/html",
+		("text/html; charset=UTF-8",
 		 HtmlTemplates.page ~body:pre
 		   ~title:(sprintf "Search results for '%s'" 
 			     (String.concat ~sep:" " request.search))
@@ -302,8 +348,8 @@ struct
 	    loop tl (keystring::keystrings)
 	  with
 	      e ->
-		ignore (eplerror 2 e "Error fetching key from hash %s" 
-			  (KeyHash.hexify hash));
+		eplerror 2 e "Error fetching key from hash %s" 
+		(KeyHash.hexify hash);
 		loop tl keystrings
     in
     loop hashes []
@@ -339,11 +385,20 @@ struct
       Filename.concat !Settings.basedir (Filename.concat "web" fname)
     else raise (Wserver.Misc_error "Malformed requst")
 
+  let supported_extensions = 
+    [ ".jpg", "image/jpeg";
+      ".gif", "image/gif";
+      ".ico", "image/x-icon";
+      ".png", "image/png";
+      ".txt", "text/plain"; 
+    ]
+
   (** Handler for HTTP requests *)
   let webhandler addr msg cout = 
     match msg with 
       | Wserver.GET (request,headers) ->
-	  ignore (plerror 5 "Get request received");
+	  plerror 5 "Get request: %s => %s" 
+	    (sockaddr_to_string addr) request;
 	  let (base,oplist) = string_to_oplist request in
 	  if base = "/pks/lookup" then (
 	    let request = request_of_oplist oplist in
@@ -357,23 +412,21 @@ struct
 	      let fname = convert_web_fname "index.html" in 
 	      let text = read_file fname in
 	      cout#write_string text;
-	      "text/html"
+	      "text/html; charset=UTF-8"
 	    else 
 	      (try 
 		 let extension = get_extension base in
-		 if extension = ".jpg" || extension = ".gif" || extension = ".png"
-		 then 
-		   let base = base </> (1,0) in
-		   let image = read_file ~binary:true (convert_web_fname base) in
-		   cout#write_string image;
-		   (match extension with
-		      | ".jpg" -> "image/jpeg" | ".gif" -> "image/gif"
-		      | ".png" -> "image/png" 
-		      | _ -> raise (Wserver.Misc_error 
-				      ("internal error: no mimetype " ^
-				       "for image extension"))
-		   )
-		 else raise Not_found
+		 let mimetype = 
+		   try List.assoc extension supported_extensions
+		   with Not_found -> 
+		     raise (Wserver.Misc_error 
+			      ("internal error: no mimetype " ^
+			       "for given extension"))
+		 in
+		 let base = base </> (1,0) in
+		 let data = read_file ~binary:true (convert_web_fname base) in
+		 cout#write_string data;
+		 mimetype
 	       with
 		   Not_found -> raise (Wserver.Page_not_found base)
 	      )
@@ -382,50 +435,56 @@ struct
 	  let request = Wserver.strip request in
 	  match request with
 	      "/pks/add" ->
-		let keytext = Scanf.sscanf body "keytext=%s" (fun s -> s) in
-		let keytext = Wserver.decode keytext in
+		let body = Wserver.decode body in
+		if body </> (0,8) <> "keytext=" then failwith "Malformed query";
+		let keytext = body </> (8,0) in
 		let keys = Armor.decode_pubkey keytext in
-		ignore (plerror 3 "Handling /pks/add for %d keys" 
-			  (List.length keys)); 
+		plerror 3 "Handling /pks/add for %d keys" 
+		  (List.length keys); 
 		cout#write_string "<html><body>";
+		let ctr = ref 0 in
 		List.iter keys
 		  ~f:(fun origkey -> 
 			try
 			  let key = Fixkey.canonicalize origkey in
-			  signore (plerror 3 "/pks/add: key %s added to database"
-				     (KeyHash.hexify (KeyHash.hash key))
-				  );
+			  plerror 3 "/pks/add: key %s added to database"
+			    (KeyHash.hexify (KeyHash.hash key));
 			  Keydb.add_key_merge ~newkey:true key;
-			  cout#write_string "Key Accepted<br>"
+			  incr ctr;
 			with
-			  | KeyMerge.Unparseable_packet_sequence ->
+			  | Fixkey.Bad_key | KeyMerge.Unparseable_packet_sequence ->
 			      cout#write_string
 			      ("Add failed: Malformed Key --- unexpected packet " ^
-			       "type and/or order of packets");
-			      signore 
-				(plerror 2 "key %s %s"
-				   (KeyHash.hexify (KeyHash.hash origkey))
-				   "could not be parsed by KeyMerge.canonicalize")
+			       "type and/or order of packets<br>");
+			      plerror 2 "key %s %s"
+				(KeyHash.hexify (KeyHash.hash origkey))
+				"could not be parsed by KeyMerge.canonicalize"
 			  | Bdb.Key_exists as e ->
 			      cout#write_string 
 			      ("Add failed: identical key already " ^
 			       "exists in database<br>");
-			      signore (eperror e "Key add failed")
+			      eperror e "Key add failed"
 			  | e -> 
 			      Eventloop.reraise e;
 			      cout#write_string "Add failed<br>"; 
-			      signore (eperror e "Key add failed")
+			      eperror e "Key add failed"
 		     );
+		if !ctr > 0 then (
+		  cout#write_string 
+		    ("Key block added to key server database.\n  " ^
+		     "New public keys added: <br>");
+		  cout#write_string (sprintf "%d keys added succesfully.<br>" !ctr)
+		);
 		cout#write_string "</html></body>";
-		"text/html"
+		"text/html; charset=UTF-8"
 	    | "/pks/hashquery" ->
-		ignore (plerror 4 "Handling /pks/hashquery"); 
+		plerror 4 "Handling /pks/hashquery"; 
 		let sin = new Channel.string_in_channel body 0 in
 		let hashes = 
 		  CMarshal.unmarshal_list ~f:CMarshal.unmarshal_string sin
 		in
 		let keystrings = get_keystrings_from_hashes hashes in
-		signore (perror "%d keys found" (List.length keystrings));
+		perror "%d keys found" (List.length keystrings);
 		CMarshal.marshal_list ~f:CMarshal.marshal_string cout 
 		  keystrings;
 		"pgp/keys" (* This is a bogus content-type *)
@@ -433,7 +492,7 @@ struct
 		cout#write_string (HtmlTemplates.page 
 				     ~title:"Unexpected POST request" 
 				     ~body:"");
-		"text/html"
+		"text/html; charset=UTF-8"
 
 
   (** Prepare handler for use with eventloop by transforming system
@@ -462,11 +521,11 @@ struct
 	  let logresp = Keydb.logquery ~maxsize:count timestamp in
 	  let length = List.length logresp in
 	  if length > 0 then
-	    ignore (plerror 3 "Sending LogResp size %d" length);
+	    plerror 3 "Sending LogResp size %d" length;
 	  marshal cout (LogResp logresp)
 
       | WordQuery words -> 
-	  ignore (plerror 3 "Handling WordQuery");
+	  plerror 3 "Handling WordQuery";
 	  let keys = Keydb.get_by_words ~max:!Settings.max_matches words in
 	  marshal cout (Keys keys)
 
@@ -474,15 +533,15 @@ struct
 	  let keys = List.fold_left ~init:[] keys
 		       ~f:(fun list key ->
 			     try (Fixkey.canonicalize key)::list
-			     with KeyMerge.Unparseable_packet_sequence -> list
+			     with KeyMerge.Unparseable_packet_sequence | Fixkey.Bad_key -> list
 			  )
 	  in
 	  marshal cout (Ack 0);
 	  (try Keydb.add_keys_merge keys
-	   with e -> ignore (eplerror 2 e "Key addition failed"))
+	   with e -> eplerror 2 e "Key addition failed")
 
       | DeleteKey hash ->
-	  ignore (plerror 3 "Handling DeleteKey");
+	  plerror 3 "Handling DeleteKey";
 	  ( try
 	      let hash = RMisc.truncate hash KeyHash.hash_bytes in
 	      let key = Keydb.get_by_hash hash in
@@ -494,40 +553,40 @@ struct
 		  raise e
 	  )
       | HashRequest hashes ->
-	  ignore (plerror 3 "Handling HashRequest");
+	  plerror 3 "Handling HashRequest";
 	  let keys = 
 	    List.fold_left hashes ~init:[]
 	      ~f:(fun list hash ->
 		    try (Keydb.get_by_hash hash)::list
 		    with 
 			Not_found -> 
-			  ignore (plerror 2 "Requested key %s not found"
-				    (Utils.hexstring hash));
+			  plerror 2 "Requested key %s not found"
+			  (Utils.hexstring hash);
 			  list
 		 )
 	  in
-	  ignore (plerror 3 "Returning set of %d keys" (List.length keys));
+	  plerror 3 "Returning set of %d keys" (List.length keys);
 	  marshal cout (Keys keys)
 
 
       | Config (s,cvar) ->
-	  ignore (plerror 4 "Received config message");
+	  plerror 4 "Received config message";
 	  (match (s,cvar) with
 	     | ("checkpoint", `none) ->
 		 checkpoint ()
 	     | ("filters", `none) ->
 		 marshal cout (Filters (get_filters ()))
 	     | (str,value) ->
-		 ignore (perror "Unexpected config request <%s>" str)
+		 perror "Unexpected config request <%s>" str
 	  )
 	  
 
       | m -> 
 	  marshal cout ProtocolError;
-	  ignore (perror "Unexpected (%s) message" (msg_to_string m))
+	  perror "Unexpected (%s) message" (msg_to_string m)
 
 
-  (****************************************************************************)
+  (***********************************************************************)
 
   (** dequeues and transmits single key.  Returns true if there 
     might be more keys to be handled. *)
@@ -550,18 +609,18 @@ struct
 			    "Subject","incremental";
 			    "Precedence","list";
 			    "Content-type", "application/pgp-keys";
-			    "X-KeyServer-Sent",!Settings.hostname;
+                            "X-KeyServer-Sent", Settings.get_from_addr ();
 			  ] ;
 			Sendmail.body = body;
 		      }
 	    in
 	    let string = Sendmail.msg_to_string msg in
-	    signore (plerror 3 "Message transmitted for key %s"
-		       (KeyHash.hexify (KeyHash.hash key)));
-	    signore (plerror 6 "%s" string);
+	    plerror 3 "Message transmitted for key %s"
+	      (KeyHash.hexify (KeyHash.hash key));
+	    plerror 6 "%s" string;
 	    Sendmail.send msg;
 	    Keydb.txn_commit txn;
-	    signore (plerror 5 "transmission queue transaction committed");
+	    plerror 5 "transmission queue transaction committed";
 	    true
 	| None -> 
 	    (* nothing was done, so commiting and aborting are same here *)
@@ -578,14 +637,14 @@ struct
     while transmit_single_key () do () done;
     []
 
-  (****************************************************************************)
+  (***********************************************************************)
 
   let run () = 
     Keydb.open_dbs settings;
     if !Settings.initial_stat then ignore (calculate_stats_page ());
-    ignore (plerror 2 "Database opened");
-    ignore (plerror 0 "Applied filters: %s" (String.concat ~sep:", " 
-					       (get_filters ())));
+    plerror 2 "Database opened";
+    plerror 0 "Applied filters: %s" (String.concat ~sep:", " 
+				       (get_filters ()));
     Eventloop.evloop
 
       (
@@ -593,41 +652,58 @@ struct
 	 then (Ehandlers.repeat_forever_simple checkpoint_interval checkpoint)
 	 else (Ehandlers.repeat_forever_simple sync_interval sync)) 
 	@
+	Ehandlers.repeat_forever_simple !Settings.membership_reload_time
+	 Membership.reset_membership_time
+	@
 	(Ehandlers.repeat_forever 10. 
 	   (Eventloop.make_tc ~cb:transmit_keys ~timeout:0
 	      ~name:"mail transmit keys" )
 	)
 	@
-	(Ehandlers.repeat_forever 10. 
-	   (Eventloop.make_tc ~name:"mailsync" ~timeout:0
-	      ~cb:(Mailsync.load_mailed_keys 
-		     ~addkey:(Keydb.add_key_merge ~newkey:false)))
-	)
+	(if !Settings.send_mailsyncs then
+	   (Ehandlers.repeat_forever 10. 
+	      (Eventloop.make_tc ~name:"mailsync" ~timeout:0
+		 ~cb:(Mailsync.load_mailed_keys 
+			~addkey:(Keydb.add_key_merge ~newkey:false)))
+	   )
+	 else [])
 	@
 	(Ehandlers.repeat_at_hour !Settings.stat_calc_hour
 	   calculate_stats_page)
       )
 
-      [(websock, Eventloop.make_th ~name:"webserver" 
+      (
+	(websock, Eventloop.make_th ~name:"webserver" 
 	  ~timeout:!Settings.wserver_timeout
-	  ~cb:(Wserver.accept_connection webhandler ~recover_timeout:1)
-       );
-       (comsock, Eventloop.make_th ~name:"command handler" 
-	  ~timeout:!Settings.command_timeout
-	  ~cb:(eventify_handler command_handler)
-       );
-      ]
+	  ~cb:(Wserver.accept_connection webhandler ~recover_timeout:1))
+	::
+	 (comsock, Eventloop.make_th ~name:"command handler" 
+	    ~timeout:!Settings.command_timeout
+	    ~cb:(eventify_handler command_handler))
+	::
+	 (if !Settings.use_port_80 then
+	    let sock = Eventloop.create_sock (ADDR_INET (addr,80)) in
+	    (sock,Eventloop.make_th ~name:"webserver80" 
+	       ~timeout:!Settings.wserver_timeout
+	       ~cb:(Wserver.accept_connection webhandler ~recover_timeout:1)
+	    )::[]
+	  else
+	    []
+	 )
+      )
+
+
 
   let run () = 
     protect ~f:run
       ~finally:(fun () -> 
 		  set_catch_break false;
-		  ignore (plerror 0 "Shutting down database"); 
+		  plerror 0 "Shutting down database"; 
 		  Keydb.sync ();
-		  ignore (plerror 0 "Database sync'd"); 
+		  plerror 0 "Database sync'd"; 
 		  Keydb.unconditional_checkpoint ();
-		  ignore (plerror 0 "Database checkpointed"); 
+		  plerror 0 "Database checkpointed"; 
 		  Keydb.close_dbs ();
-		  ignore (plerror 0 "Database closed")
+		  plerror 0 "Database closed"
 	       )
 end
