@@ -144,22 +144,64 @@ let sig_to_siginfo sign =
 
 (********************************************************************)
 
+(** sort signatures in ascending time order *)
 let sort_siginfo_list list = 
-  List.sort list
+  List.stable_sort list
     ~cmp:(fun x y -> compare x.sig_creation_time y.sig_creation_time)
-    (* compare *)
 
 (********************************************************************)
 
-let is_primary (uid,siginfo_list) =
+let is_selfsig ~keyid siginfo = siginfo.keyid = Some keyid
+
+(********************************************************************)
+
+let is_primary ~keyid (uid,siginfo_list) =
   List.exists ~f:(fun siginfo -> 
-		    siginfo.is_primary_uid 
+		    is_selfsig ~keyid siginfo
+		    && siginfo.is_primary_uid 
 		    && uid.packet_type = User_ID_Packet
 		 )
     siginfo_list
 
 (********************************************************************)
 
+(** returns time of most recent self-sig on uid *)
+let max_selfsig_time ~keyid (uid,siginfo_list) = 
+  let selfsigs = List.filter ~f:(fun si -> is_selfsig ~keyid si) 
+		   siginfo_list in
+  let times = filter_opts
+		(List.map siginfo_list
+		   ~f:(function x -> match x.sig_creation_time with
+			   None -> None
+			 | Some time -> Some (Int64.to_float time)))
+  in
+  List.fold_left ~init:min_float ~f:max times
+
+(********************************************************************)
+
+let split_list ~f l = 
+  let rec loop l a b = match l with
+      [] -> (List.rev a, List.rev b)
+    | hd::tl ->
+	if f hd then loop tl (hd::a) b
+	else loop tl a (hd::b)
+  in
+  loop l [] []
+
+(********************************************************************)
+
+let move_primary_to_front ~keyid uids = 
+  let (primary,normal) = split_list ~f:(is_primary ~keyid) uids in
+  let primary = List.stable_sort primary
+	       ~cmp:(fun x y -> compare
+		       (max_selfsig_time ~keyid y)
+		       (max_selfsig_time ~keyid x)
+		    )
+  in
+  primary @ normal
+
+(********************************************************************)
+
 let convert_sigpair (uid,sigs) = 
   (uid,List.map ~f:sig_to_siginfo sigs)
 
@@ -169,7 +211,8 @@ let blank_datestr = "__________"
 let no_datestr =    "          "
 let datestr_of_int64 i = 
   let tm = Unix.gmtime (Int64.to_float i) in
-  sprintf "%04d-%02d-%02d" (1900 + tm.Unix.tm_year) (1 + tm.Unix.tm_mon) (tm.Unix.tm_mday)
+  sprintf "%04d-%02d-%02d" (1900 + tm.Unix.tm_year) (1 + tm.Unix.tm_mon) 
+    (tm.Unix.tm_mday)
 
 (********************************************************************)
 
@@ -202,7 +245,7 @@ let siginfo_to_lines ~get_uid ?key_creation_time request self_keyid siginfo =
       | 0x11 -> " sig1"
       | 0x12 -> " sig2"
       | 0x13 -> " sig3"
-      | 0x20 | 0x28 | 0x30 -> "<font color=\"red\"><b>revok</b></font>"
+      | 0x20 | 0x28 | 0x30 -> "<font color=\"red\"><b>revok </b></font>"
       | 0x1f -> "dirct"
       | 0x18 -> "sbind"
       | x -> sprintf " 0x%02x" x
@@ -226,9 +269,7 @@ let siginfo_to_lines ~get_uid ?key_creation_time request self_keyid siginfo =
 	  let long = Fingerprint.keyid_to_string ~short:false keyid in
 	  let link = 
 	    HtmlTemplates.link ~op:"vindex" 
-	      ~hash:request.hash ~fingerprint:request.fingerprint
-	      ~hostname:!Settings.hostname
-	      ~port:http_port ~keyid:long
+	      ~hash:request.hash ~fingerprint:request.fingerprint ~keyid:long
 	  in
 	  sprintf "<a href=\"%s\">%s</a>" link uid_string
   in
@@ -239,16 +280,14 @@ let siginfo_to_lines ~get_uid ?key_creation_time request self_keyid siginfo =
 	let long = Fingerprint.keyid_to_string ~short:false keyid in
 	let link = 
 	  HtmlTemplates.link ~op:"get" 
-	    ~hash:request.hash ~fingerprint:request.fingerprint 
-	    ~hostname:!Settings.hostname
-	    ~port:http_port ~keyid:long
+	    ~hash:request.hash ~fingerprint:request.fingerprint ~keyid:long
 	in
 	sprintf "<a href=\"%s\">%s</a>" link short
     | None -> 
 	"no keyid"
   in
 
-  let firstline = sprintf "sig %s %s %s %s %s %s"
+  let firstline = sprintf "sig %-6s %s %s %s %s %s"
 		    sigtype_string keyid_string
 		    sig_creation_string sig_expiration_string 
 		    key_expiration_string
@@ -259,7 +298,8 @@ let siginfo_to_lines ~get_uid ?key_creation_time request self_keyid siginfo =
     apply_opt siginfo.policy_url
       ~f:(fun policy_url -> 
 	    let policy_url = HtmlTemplates.html_quote policy_url in
-	    sprintf "    Policy URL: <a href=\"%s\">%s</a>" policy_url policy_url
+	    sprintf "    Policy URL: <a href=\"%s\">%s</a>" 
+	      policy_url policy_url
 	 )
   in
   let notation_data_opt = 
@@ -274,14 +314,15 @@ let siginfo_to_lines ~get_uid ?key_creation_time request self_keyid siginfo =
     apply_opt siginfo.revocation_key 
       ~f:(fun fingerprint ->
 	    sprintf "    Revocation key fingerprint: <a href=\"%s\">%s</a>"
-	    (HtmlTemplates.link ~hash:request.hash ~op:"vindex" ~hostname:!Settings.hostname
-	       ~port:http_port ~fingerprint:request.fingerprint 
+	    (HtmlTemplates.link ~hash:request.hash ~op:"vindex" 
+	       ~fingerprint:request.fingerprint 
 	       ~keyid:(Utils.hexstring fingerprint)
 	    )
 	    (Fingerprint.fp_to_string fingerprint)
 	 )
   in
-  firstline :: filter_opts [policy_url_opt; notation_data_opt; revocation_key_opt]
+  firstline :: filter_opts [policy_url_opt; notation_data_opt;
+			    revocation_key_opt]
 
 
 (********************************************************************)
@@ -297,19 +338,22 @@ let selfsigs_to_lines request key_creation_time keyid selfsigs =
 
 (********************************************************************)
 
-let uid_to_lines ~get_uid request key_creation_time keyid (uid,siginfo_list) = 
+let uid_to_lines ~get_uid request key_creation_time keyid 
+  (uid,siginfo_list) = 
   let siginfo_list = sort_siginfo_list siginfo_list in
   let uid_line = match uid.packet_type with
     | User_ID_Packet -> 
 	sprintf "<b>uid</b> <font color=\"green\"><u>%s</u></font>" 
 	(HtmlTemplates.html_quote uid.packet_body)
+
     | _ -> sprintf "<b>uat</b> [contents omitted]"
   in
   let creation_string = datestr_of_int64 in
   let siginfo_lines = 
     List.concat 
-      (List.map ~f:(siginfo_to_lines ~get_uid ~key_creation_time request keyid)
-	 siginfo_list)   
+      (List.map ~f:(siginfo_to_lines ~get_uid ~key_creation_time 
+		      request keyid)
+	 siginfo_list)
   in
   ""::uid_line::siginfo_lines
 
@@ -341,13 +385,13 @@ let key_packet_to_line ~is_subkey pki keyid =
     else
       sprintf "<a href=\"%s\">%8s</a>"
 	(HtmlTemplates.link ~op:"get" ~hash:false ~fingerprint:false
-	   ~hostname:!Settings.hostname
-	   ~port:http_port ~keyid:keyid_long ) 
+	   ~keyid:keyid_long ) 
 	keyid_short
   in
-  let line = sprintf "%s  %4d/%s %s %s "
+  let algo = pk_alg_to_ident pki.pk_alg in
+  let line = sprintf "%s  %4d%s/%s %s %s "
 	       prefix
-	       pki.pk_keylen 
+	       pki.pk_keylen algo
 	       keyid_string
 	       creation_string expiration_string
   in
@@ -363,7 +407,8 @@ let subkey_to_lines request (subkey,siginfo_list) =
   let siginfo_lines = 
     List.concat (List.map ~f:(siginfo_to_lines ~get_uid:(fun _ -> None)
 				~key_creation_time request keyid) 
-		   siginfo_list) in
+		   siginfo_list) 
+  in
   ""::subkey_line::siginfo_lines
 
 let subkeys_to_lines request subkeys = 
@@ -373,8 +418,9 @@ let subkeys_to_lines request subkeys =
 (* new style verbose key index **************************************)
 (********************************************************************)
 
-(** if f is true for any element of list, then return (Some x,newlist), where x is 
-  one such element, and newlist is list with x removed.  Otherwise, return (None,list)
+(** if f is true for any element of list, then return (Some x,newlist), where
+  x is one such element, and newlist is list with x removed.  Otherwise,
+  return (None,list)
 *)
 let rec extract ~f list = match list with
     [] -> (None,[])
@@ -403,21 +449,21 @@ let get_uid get_uids =
 	 let uids = List.filter uids
 		      ~f:(fun (uid,_) -> uid.packet_type = User_ID_Packet) in
 	 let uids = List.map ~f:convert_sigpair uids in
-	 match move_to_front ~f:is_primary uids with
+	 match move_primary_to_front ~keyid uids with
 	   | [] -> None
 	   | (uid,_)::tl -> Some uid.packet_body
-	 with
-	     e -> 
-	       signore (eplerror 3 e 
-			  "Error fetching uid during VIndex for keyid 0x%s"
-			  (KeyHash.hexify keyid));
-	       None
+     with
+       | e -> 
+	   eplerror 3 e 
+	     "Error fetching uid during VIndex for keyid 0x%s"
+	     (KeyHash.hexify keyid);
+	   None
   )
   
 (********************************************************************)
 
 (** computes fingerprint and hash lines if required *)
-let get_extra_lines request key meta = 
+let get_extra_lines request key hash meta = 
   
   let extra_lines = 
     if request.fingerprint then
@@ -428,23 +474,18 @@ let get_extra_lines request key meta =
 
   let extra_lines = 
     if request.hash then
-      let hash_line = HtmlTemplates.hash 
-			~hostname:!Settings.hostname
-			~port:http_port
-			~hash:(KeyHash.hexify (KeyHash.hash key))
-      in
+      let hash_line = HtmlTemplates.hash ~hash:(KeyHash.hexify hash) in
       hash_line::extra_lines
     else 
       extra_lines
   in
-
   extra_lines
 
 (********************************************************************)
 
 (** computes key to verbose set of lines.  Note that these lines should be
   embedded inside of a <pre></pre> environment *)
-let key_to_lines_verbose ~get_uids request key = 
+let key_to_lines_verbose ~get_uids request key hash = 
   try
     let get_uid = get_uid get_uids in
     let pkey = KeyMerge.key_to_pkey key in
@@ -453,15 +494,10 @@ let key_to_lines_verbose ~get_uids request key =
     and subkeys = List.map ~f:convert_sigpair pkey.KeyMerge.subkeys
     and pubkey = pkey.KeyMerge.key in
 
-    (* sort subkeys by creation time in descending order *)
+    (* sort subkeys by creation time in ascending order *)
     let subkeys = 
-      List.map ~f:(fun (uid,siginfo) -> (uid,sort_siginfo_list siginfo)) subkeys
-    in
-
-    (** move primary keyid to front of the list *)
-    let uids = 
-      List.sort uids
-	~cmp:(fun x y -> compare (is_primary y,fst y) (is_primary x,fst x))
+      List.map ~f:(fun (uid,siginfo) -> 
+		     (uid,sort_siginfo_list siginfo)) subkeys
     in
 
     let pki = ParsePGP.parse_pubkey_info pubkey in
@@ -469,30 +505,35 @@ let key_to_lines_verbose ~get_uids request key =
     let keyid = meta.Fingerprint.keyid in
     let key_creation_time = pki.pk_ctime in
 
-    (* let primary_uid_string = (fst (List.hd uids)).packet_body in *)
-    let (pubkey_line,keyid) = key_packet_to_line ~is_subkey:false pki keyid in
 
-    let extra_lines = get_extra_lines request key meta in
+    (** move primary keyid to front of the list *)
+    let uids = move_primary_to_front ~keyid uids in
+
+    (* let primary_uid_string = (fst (List.hd uids)).packet_body in *)
+    let (pubkey_line,keyid) = 
+      key_packet_to_line ~is_subkey:false pki keyid in
+
+    let extra_lines = get_extra_lines request key hash meta in
 
     (* note: ugly hack here.  </pre> and <pre> are used to allow for an <hr>
-       inside of a pre-formatted region.  So this code only works if the lines are
-       being generated to be put inside of a <pre></pre> block> *)
+       inside of a pre-formatted region.  So this code only works if the
+       lines are being generated to be put inside of a <pre></pre> block> *)
     ("</pre><hr><pre>" ^ pubkey_line) ::
-    List.concat [
-      selfsigs_to_lines request key_creation_time keyid selfsigs;
-      extra_lines;
-      uids_to_lines ~get_uid request key_creation_time keyid uids;
-      subkeys_to_lines request subkeys;
-    ]
+      List.concat [
+	selfsigs_to_lines request key_creation_time keyid selfsigs;
+	extra_lines;
+	uids_to_lines ~get_uid request key_creation_time keyid uids;
+	subkeys_to_lines request subkeys;
+      ]
 
   with
     | Sys.Break | Eventloop.SigAlarm as e -> raise e
     | e ->
-	ignore (eplerror 2 e 
-		  "Unable to print key from query '%s'"
-		  (String.concat ~sep:" " request.search));
+	eplerror 2 e 
+	  "Unable to print key from query '%s'"
+	  (String.concat ~sep:" " request.search);
 	[]
-	
+	  
 
 
 
@@ -500,30 +541,52 @@ let key_to_lines_verbose ~get_uids request key =
 (* old style key index **********************************************)
 (********************************************************************)
 
-(** oldstyle index lines *)
-let key_to_lines_normal request key = 
-  try
-    let meta = Fingerprint.from_key key in
-    let keyid_short = Fingerprint.keyid_to_string ~short:true 
-			meta.Fingerprint.keyid 
-    in
-    let keyid_long = Fingerprint.keyid_to_string ~short:false 
-		       meta.Fingerprint.keyid 
-    in
-    let link = HtmlTemplates.link ~op:"get" ~hash:false ~fingerprint:false
-		 ~hostname:!Settings.hostname
-		 ~port:http_port ~keyid:keyid_long in
-    let ilink = HtmlTemplates.link ~op:"vindex" 
-		  ~hash:request.hash ~fingerprint:request.fingerprint
-		 ~hostname:!Settings.hostname
-		 ~port:http_port ~keyid:keyid_long in
+let sig_is_revok siginfo =
+  match siginfo.sigtype with
+    | 0x20 | 0x28 | 0x30 -> true
+    | _ -> false
 
-    let userids = List.map ~f:HtmlTemplates.html_quote (Key.get_ids key) in
+let is_revoked key = 
+  let pkey = KeyMerge.key_to_pkey key in
+  let selfsigs = pkey.KeyMerge.selfsigs in
+  List.exists ~f:(fun sign -> 
+                   sig_is_revok (sig_to_siginfo sign)
+                 )
+    selfsigs
+
+(** oldstyle index lines *)
+let key_to_lines_normal request key hash = 
+  try
+    let pkey = KeyMerge.key_to_pkey key in
+    let uids = List.map ~f:convert_sigpair pkey.KeyMerge.uids in
+
+    let meta = Fingerprint.from_key key in
+    let keyid = meta.Fingerprint.keyid in
+    let keyid_short = Fingerprint.keyid_to_string ~short:true keyid in
+    let keyid_long = Fingerprint.keyid_to_string ~short:false keyid in
+    let link = HtmlTemplates.link ~op:"get" ~hash:false ~fingerprint:false 
+		 ~keyid:keyid_long in
+    let ilink = HtmlTemplates.link ~op:"vindex" 
+		  ~hash:request.hash ~fingerprint:request.fingerprint 
+		  ~keyid:keyid_long in
+
+    let uids = move_primary_to_front ~keyid uids in
+
+    let userids = 
+      List.map ~f:(fun (uid,sigs) -> 
+		     match uid.packet_type with
+			 User_ID_Packet -> 
+			   HtmlTemplates.html_quote uid.packet_body
+		       | User_Attribute_Packet -> "[user attribute packet]"
+		       | _ -> "[unexpected packet type]"
+		  )
+	uids 
+    in
     let userids = match userids with [] -> []
       | hd::tl -> (sprintf "<a href=\"%s\">%s</a>" ilink hd)::tl in
     let pki = ParsePGP.parse_pubkey_info (List.hd key) in
-    let keystr = HtmlTemplates.keyinfo_pks pki 
-		   ~keyid:keyid_short ~link ~userids in
+    let keystr = HtmlTemplates.keyinfo_pks pki (is_revoked key) 
+ 		   ~keyid:keyid_short ~link ~userids in
     let lines = [] in
     let lines = 
       if request.fingerprint then
@@ -537,12 +600,7 @@ let key_to_lines_normal request key =
     in
     let lines = 
       if request.hash then
-	let hash = HtmlTemplates.hash 
-		     ~hostname:!Settings.hostname
-		     ~port:http_port
-		     ~hash:(KeyHash.hexify 
-			      (KeyHash.hash key))
-	in
+	let hash = HtmlTemplates.hash ~hash:(KeyHash.hexify hash) in
 	hash::lines
       else 
 	lines
@@ -551,9 +609,9 @@ let key_to_lines_normal request key =
   with
     | Sys.Break | Eventloop.SigAlarm as e -> raise e
     | e ->
-	ignore (eplerror 2 e 
-		  "Unable to print key from query '%s'"
-		  (String.concat ~sep:" " request.search));
+	eplerror 2 e 
+	  "Unable to print key from query '%s'"
+	  (String.concat ~sep:" " request.search);
 	[]
 
 
