@@ -28,6 +28,7 @@ open Common
 
 exception Bug of string
 exception Lookup_failure of string
+exception Malformed_entry of string
 
 let membership = ref ([| |],-1.)
 
@@ -46,32 +47,30 @@ let local_recon_addr = Utils.unit_memoize local_recon_addr
 let remove_self addresses = 
   List.filter ~f:(fun (addr,str) -> addr <> local_recon_addr ()) addresses
 
-let safe_readline file =
-  try
-    let l = decomment (input_line file) in
-    Some l
-  with
-      End_of_file -> None
-
-let safe_convert_address l =
-  try
-    let entry =
-      sscanf l "%s %d" 
-	(fun addr port -> Unix.ADDR_INET (lookup_hostname addr,port))
-    in
-    Some entry
-  with
-      | End_of_file | Scanf.Scan_failure _ | Failure _ | Not_found -> None
+let convert_address l =
+  try 
+    sscanf l "%s %d" 
+    (fun addr port -> Unix.ADDR_INET (lookup_hostname addr,port))
+  with 
+    Scanf.Scan_failure _ | End_of_file | Failure _ -> raise (Malformed_entry l)
 
 let load_membership_file file =
   let rec loop list =
-    match safe_readline file with
-      | None -> list
-      | Some l -> match safe_convert_address l with
-	    | Some e -> loop ((e,l)::list)
-	    | None ->
-		ignore (perror "Unknown host in membership file: %s" l);
-		loop list
+    try
+      let line = Wserver.strip (decomment (input_line file)) in
+      if String.length line > 0 then
+        let addr = convert_address line in
+        (addr,line) :: loop list
+      else
+        loop list
+    with
+      | End_of_file -> list
+      | Lookup_failure addr -> 
+	  perror "Lookup failure on address %s" addr;
+	  loop list
+      | Malformed_entry line -> 
+	  perror "Malformed entry %s" line;
+	  loop list
   in
   let raw_membership = loop [] in
   Array.of_list (remove_self raw_membership)
@@ -90,9 +89,9 @@ let load_membership fname =
 		let mshp = load_membership_file file in
 		match get_mtime fname with
 		  | None -> 
-		      ignore (plerror 2 "%s" 
-				("Unable to get mtime for membership. " ^
-				 "Failed to reload."))
+		      plerror 2 "%s" 
+			("Unable to get mtime for membership. " ^
+			 "Failed to reload.")
 		  | Some mtime -> membership := (mshp,mtime)
 	     )
     ~finally:(fun () -> close_in file)
@@ -112,21 +111,21 @@ let membership_string () =
     
 
 let reload_if_changed () = 
-  let fname = !Settings.membership_file in
+  let fname = Lazy.force Settings.membership_file in
   let (mshp,old_mtime) = !membership in
   match get_mtime fname with
-    | None -> signore 
-	(plerror 2 "%s" ("Unable to get mtime for membership file. " ^
-			 "Can't decide whether to reload"))
+    | None -> 
+	plerror 2 "%s" ("Unable to get mtime for membership file. " ^
+			"Can't decide whether to reload")
     | Some mtime ->
 	if old_mtime <> mtime then 
 	  ( load_membership fname;
-	    ignore (plerror 5 "%s" (membership_string ()))
+	    plerror 5 "%s" (membership_string ())
 	  )
 
 let get_names () = 
   let mshp = 
-    if Sys.file_exists !Settings.membership_file then (
+    if Sys.file_exists (Lazy.force Settings.membership_file) then (
       reload_if_changed ();
       let (m,mtime) = !membership in 
       m
@@ -136,9 +135,13 @@ let get_names () =
   Array.map ~f:snd mshp
 
 
+let reset_membership_time () =
+  let (m,mtime) = !membership in
+  membership := (m,0.)
+
 let get () = 
   let mshp = 
-    if Sys.file_exists !Settings.membership_file then (
+    if Sys.file_exists (Lazy.force Settings.membership_file) then (
       reload_if_changed ();
       let (m,mtime) = !membership in 
       m
@@ -188,23 +191,23 @@ let load_mailsync_partners fname =
     match get_mtime fname with
       | Some mtime -> 
 	  mailsync_partners := (load_mailsync_partners_file file,mtime)
-      | None -> signore 
-	  (plerror 2 "Failed to find mtime -- can't load mailsync file")
+      | None -> 
+	  plerror 2 "Failed to find mtime -- can't load mailsync file"
   in
   protect ~f:run ~finally:(fun () -> close_in file)
 
 let reload_mailsync_if_changed () = 
-  let fname = !Settings.mailsync_file in
+  let fname = Lazy.force Settings.mailsync_file in
   let (mshp,old_mtime) = !membership in
   match get_mtime fname with
-      None -> signore (plerror 2 "%s" 
-			 ("Failed to find mtime, can't decide whether to" ^
-			  " load mailsync file"))
+      None -> plerror 2 "%s" 
+	("Failed to find mtime, can't decide whether to" ^
+	 " load mailsync file")
     | Some mtime -> if old_mtime <> mtime then load_mailsync_partners fname
 
 let get_mailsync_partners () = 
   let partners = 
-    if Sys.file_exists !Settings.membership_file then (
+    if Sys.file_exists (Lazy.force Settings.membership_file) then (
       reload_mailsync_if_changed ();
       let (m,mtime) = !mailsync_partners in
       m
