@@ -90,9 +90,6 @@ let set_max_outstanding_recon_requests value =
 let max_uid_fetches = ref 1000
 let set_max_uid_fetches value = max_uid_fetches := value
 
-let basedir = ref "."
-let set_basedir value = basedir := value
-
 let dump_new = ref false 
 
 (* whether or not to use a disk-based prefix-tree implementation *)
@@ -180,7 +177,13 @@ let sendmail_cmd = ref "sendmail -t -oi"
 let set_sendmail_cmd value = sendmail_cmd := value
 
 let membership_reload_time = ref (60. *. 60. *. 6.)
-let set_membership_reload_time value = membership_reload_time := value *. 60. *. 60.
+let set_membership_reload_time value = 
+  membership_reload_time := value *. 60. *. 60.
+
+(** whether to send out PKS-style mailsync messages *)
+let send_mailsyncs = ref true
+(** WHether to log hashes of most-recently-found diff *)
+let log_diffs = ref true
 
 let from_addr = ref None
 let set_from_addr value = from_addr := Some value
@@ -194,124 +197,126 @@ let get_from_addr () =
 	from_addr := Some addr;
 	addr
 
-let dbdir = ref "KDB"
-let ptree_dbdir = ref "PTree"
-let membership_file = ref "membership"
-let mailsync_file = ref "mailsync"
-let dumpdir = ref "dump"
-let msgdir = ref "messages"
-let failed_msgdir = ref "failed_messages"
+let basedir = ref "."
 
-let update_dir dirref =
-  dirref := Filename.concat !basedir !dirref
+let base_dbdir = "KDB"
+let base_ptree_dbdir = "PTree"
+let base_membership_file = "membership"
+let base_mailsync_file = "mailsync"
+let base_dumpdir = "dump"
+let base_msgdir = "messages"
+let base_failed_msgdir = "failed_messages"
 
-let prepare_options () =
-  update_dir dbdir;
-  update_dir ptree_dbdir;
-  update_dir membership_file;
-  update_dir mailsync_file;
-  update_dir dumpdir;
-  update_dir msgdir;
-  update_dir failed_msgdir;
-  ()
+let dbdir = lazy (Filename.concat !basedir base_dbdir)
+let ptree_dbdir = lazy (Filename.concat !basedir base_ptree_dbdir)
+let membership_file = lazy (Filename.concat !basedir base_membership_file)
+let mailsync_file = lazy (Filename.concat !basedir base_mailsync_file)
+let dumpdir = lazy (Filename.concat !basedir base_dumpdir)
+let msgdir = lazy (Filename.concat !basedir base_msgdir)
+let failed_msgdir = lazy (Filename.concat !basedir base_failed_msgdir)
 
 (*****************************************************************)
 
 (** Specifies the options along with the corresponding actions.  
   These are used both for command-line options and the config file *)
 let parse_spec = 
-  [ ("-debug", Arg.Set debug, "debugging mode");
+  [ ("-debug", Arg.Set debug, " debugging mode");
     ("-debuglevel", Arg.Int set_debuglevel, 
-     "Debugging level -- sets verbosity of logging");
-    ("-q", Arg.Int set_bitquantum, "number of bits defining a bin");
-    ("-mbar", Arg.Int set_mbar, "number of errors that can be corrected " ^
+     " Debugging level -- sets verbosity of logging");
+    ("-q", Arg.Int set_bitquantum, " number of bits defining a bin");
+    ("-mbar", Arg.Int set_mbar, " number of errors that can be corrected " ^
        "in one shot");
-    ("-seed", Arg.Int set_seed, "Seed used by RNG");
-    ("-hostname", Arg.String set_hostname, "current hostname");
-    ("-d", Arg.Int set_drop, "Number of keys to drop at random " ^
+    ("-seed", Arg.Int set_seed, " Seed used by RNG");
+    ("-hostname", Arg.String set_hostname, " current hostname");
+    ("-d", Arg.Int set_drop, " Number of keys to drop at random " ^
        "when synchronizing");
-    ("-n", Arg.Int set_n, "Multiple of 15,000 keys to load at once");
+    ("-n", Arg.Int set_n, " Multiple of 15,000 keys to load at once");
     ("-max_internal_matches", Arg.Int set_max_internal_matches,
-     "Maximum number of matches for most specific word in a " ^
+     " Maximum number of matches for most specific word in a " ^
      "multi-word search");
     ("-max_matches", Arg.Int set_max_internal_matches,
-     "Maximum number of matches that will be returned from a query");
+     " Maximum number of matches that will be returned from a query");
     ("-max_uid_fetches", Arg.Int set_max_uid_fetches,
-     "Maximum number of uid fetches performed in a verbose index query");
-    ("-pagesize", Arg.Int set_pagesize, "Pagesize in bytes for key db");
-    ("-cache", Arg.Int set_cache_bytes, "Cache size in megs for key db");
+     " Maximum number of uid fetches performed in a verbose index query");
+    ("-pagesize", Arg.Int set_pagesize, " Pagesize in bytes for key db");
+    ("-cache", Arg.Int set_cache_bytes, " Cache size in megs for key db");
     ("-ptree_pagesize", Arg.Int set_ptree_pagesize, 
-     "Pagesize in bytes for prefix tree db");
+     " Pagesize in bytes for prefix tree db");
     ("-ptree_cache", Arg.Int set_ptree_cache_bytes, 
-     "Cache size in megs for prefix tree db");
-    ("-baseport",Arg.Int set_base_port, "Set base port number");
-    ("-logfile",Arg.String (fun _ -> ()), "DEPRECATED.  Now ignored.");
-    ("-recon_port",Arg.Int set_recon_port, "Set recon port number");
-    ("-recon_address",Arg.String set_recon_address, "Set recon binding address"); 
-    ("-hkp_port",Arg.Int set_hkp_port, "Set hkp port number");
-    ("-hkp_address",Arg.String set_hkp_address, "Set hkp binding address"); 
+     " Cache size in megs for prefix tree db");
+    ("-baseport",Arg.Int set_base_port, " Set base port number");
+    ("-logfile",Arg.String (fun _ -> ()), " DEPRECATED.  Now ignored.");
+    ("-recon_port",Arg.Int set_recon_port, " Set recon port number");
+    ("-recon_address",Arg.String set_recon_address, " Set recon binding address"); 
+    ("-hkp_port",Arg.Int set_hkp_port, " Set hkp port number");
+    ("-hkp_address",Arg.String set_hkp_address, " Set hkp binding address"); 
     ("-use_port_80",Arg.Set use_port_80, 
-     "Have the HKP interface listen on port 80, as well as the hkp_port"); 
-    ("-basedir", Arg.String set_basedir, "Set base directory");
+     " Have the HKP interface listen on port 80, as well as the hkp_port"); 
+    ("-basedir", Arg.Set_string basedir, " Base directory");
     ("-stdoutlog", Arg.Clear filelog, 
-     "Send log messages to stdout instead of log file");
-    ("-diskptree", Arg.Set disk_ptree, "Use a disk-based ptree " ^
+     " Send log messages to stdout instead of log file");
+    ("-diskptree", Arg.Set disk_ptree, " Use a disk-based ptree " ^
        "implementation.  "
        ^ "Slower, but requires far less memory");
-    ("-nodiskptree", Arg.Clear disk_ptree, "Use in-mem ptree");
+    ("-nodiskptree", Arg.Clear disk_ptree, " Use in-mem ptree");
     ("-max_ptree_nodes", Arg.Int set_max_ptree_nodes, 
-     "Maximum number of allowed ptree nodes.  " ^
+     " Maximum number of allowed ptree nodes.  " ^
      "Only meaningful if -diskptree is set");
-    ("-prob", Arg.Float set_prob, "Set probability.  " ^
+    ("-prob", Arg.Float set_prob, " Set probability.  " ^
        "Used for testing code only");
     ("-recon_sync_interval", Arg.Float set_recon_sync_interval, 
-     "Set sync interval for reconserver.");
-    ("-gossip_interval", Arg.Float set_gossip_interval, "Set time between " ^
+     " Set sync interval for reconserver.");
+    ("-gossip_interval", Arg.Float set_gossip_interval, " Set time between " ^
        "gossips in minutes.");
-    ("-dontgossip", Arg.Clear gossip, "Don't gossip automatically.  " ^
+    ("-dontgossip", Arg.Clear gossip, " Don't gossip automatically.  " ^
        "Host will still respond to requests from other hosts");
     ("-db_sync_interval", Arg.Float set_db_sync_interval, 
-     "Set sync interval for dbserver.");
+     " Set sync interval for dbserver.");
     ("-checkpoint_interval", Arg.Float set_checkpoint_interval, 
-     "Time period between checkpoints");
+     " Time period between checkpoints");
     ("-recon_checkpoint_interval", Arg.Float set_recon_checkpoint_interval, 
-     "Time period between checkpoints for reconserver");
+     " Time period between checkpoints for reconserver");
     ("-ptree_thresh_mult", Arg.Int set_ptree_thresh_mult, 
-     "Multiple of thresh which specifies minimum node size in prefix tree");
+     " Multiple of thresh which specifies minimum node size in prefix tree");
     ("-recon_thresh_mult", Arg.Int set_recon_thresh_mult, 
-     "Multiple of thresh which specifies minimum node size that is " ^
+     " Multiple of thresh which specifies minimum node size that is " ^
      "included in reconciliation");
     ("-max_recover", Arg.Int set_max_recover, 
-     "Maximum number of differences to recover in one round");
+     " Maximum number of differences to recover in one round");
     ("-http_fetch_size", Arg.Int set_http_fetch_size,
-     "Number of keys for reconserver to fetch from dbserver in one go.");
+     " Number of keys for reconserver to fetch from dbserver in one go.");
     ("-wserver_timeout", Arg.Int set_wserver_timeout,
-     "Timeout in seconds for webserver requests");
+     " Timeout in seconds for webserver requests");
     ("-reconciliation_timeout", Arg.Int set_reconciliation_timeout,
-     "Timeout for reconciliation runs in minutes");
+     " Timeout for reconciliation runs in minutes");
     ("-stat_hour", Arg.Int set_stat_calc_hour,
-     "Hour at which to run database statistics");
+     " Hour at which to run database statistics");
     ("-initial_stat", Arg.Set initial_stat,
-     "Runs database statistics calculation on boot");
+     " Runs database statistics calculation on boot");
     ("-reconciliation_config_timeout", Arg.Int set_reconciliation_config_timeout,
-     "Set timeout in seconds for initial exchange of config info " ^
+     " Set timeout in seconds for initial exchange of config info " ^
      "in reconciliation");
     ("-missing_keys_timeout", Arg.Int set_missing_keys_timeout,
-     "Timeout in seconds for get_missing_keys");
+     " Timeout in seconds for get_missing_keys");
     ("-command_timeout", Arg.Int set_command_timeout,
-     "Timeout in seconds for commands set over command socket");
+     " Timeout in seconds for commands set over command socket");
     ("-sendmail_cmd", Arg.String set_sendmail_cmd,
-     "Command used for sending mail");
+     " Command used for sending mail");
     ("-from_addr", Arg.String set_from_addr,
-     "From address used in synchronization emails used to communicate " ^
+     " From address used in synchronization emails used to communicate " ^
      "with PKS");
     ("-dump_new_only", Arg.Set dump_new, 
-     "When doing a database dump, only dump new keys, not keys" ^
+     " When doing a database dump, only dump new keys, not keys" ^
      " already contained in a keydump file");
     ("-max_outstanding_recon_requests", Arg.Int set_max_outstanding_recon_requests,
-     "maximum number of outstanding requests in reconciliation");
+     " maximum number of outstanding requests in reconciliation");
     ("-membership_reload_interval", Arg.Float set_membership_reload_time,
-     "maximum interval (in hours) at which membership file is reloaded");
+     " maximum interval (in hours) at which membership file is reloaded");
+    ("-disable_mailsync", Arg.Clear send_mailsyncs,
+     " Disable sending of PKS mailsync messages.  " ^
+     "ONLY FOR STANDALONE SERVERS!");
+    ("-disable_log_diffs", Arg.Clear log_diffs,
+     " Disable logging of recent hashset diffs.");
   ]
 
 let anon_options option_string = 
