@@ -29,8 +29,8 @@ struct
     Keydb.withtxn = !Settings.transactions;
     Keydb.cache_bytes = !Settings.cache_bytes;
     Keydb.pagesize = !Settings.pagesize;
-    Keydb.dbdir = !Settings.dbdir;
-    Keydb.dumpdir = !Settings.dumpdir;
+    Keydb.dbdir = Lazy.force Settings.dbdir;
+    Keydb.dumpdir = Lazy.force Settings.dumpdir;
   }
 
   module Keydb = Keydb.Unsafe
@@ -56,9 +56,8 @@ struct
 		write_to_file size stream cout
 	    with
 		e -> 
-		  signore (eplerror 1 e 
-			     "Failed attempt to extract key %s"
-			     (KeyHash.hexify hash));
+		  eplerror 1 e "Failed attempt to extract key %s" 
+		  (KeyHash.hexify hash);
 		  write_to_file size stream cout
 
 
@@ -69,12 +68,14 @@ struct
     protect ~f:(fun () -> write_to_file size stream file)
       ~finally:(fun () -> close_out file)
       
-  let dump_database dumpdir size =
+  let dump_database dumpdir size name =
     let (stream,close) = Keydb.create_hash_skey_stream () in
     let run () = 
       let ctr = ref 0 in
       while SStream.peek stream <> None do
-	let fname = Filename.concat dumpdir (sprintf "sks-dump-%04d.pgp" !ctr) in
+	let fname = 
+	  Filename.concat dumpdir (sprintf "%s-%04d.pgp" name !ctr) 
+	in
 	write_to_fname size stream fname;
 	incr ctr
       done
@@ -83,16 +84,26 @@ struct
       
 
 
+  exception Argument_error
+
   let run () = 
-    match !Settings.anonlist with
-      | [size;dumpdir] ->
-	  set_logfile ".dump";
-	  Keydb.open_dbs settings;
-	  let size = int_of_string size in
-	  dump_database dumpdir size
-      | _ ->
-	  eprintf "wrong number of arguments\n";
-	  eprintf "usage: sksdump numkeys dumpdir\n";
-	  flush stderr;
-	  exit (-1)
+    try (
+      match !Settings.anonlist with
+	| size::dumpdir::tl ->
+	    let name = match tl with
+	      | [] -> "sks-dump"
+	      | [name] -> name
+	      | _ -> raise Argument_error
+	    in
+	    set_logfile "dump";
+	    Keydb.open_dbs settings;
+	    let size = int_of_string size in
+	    dump_database dumpdir size name
+	| _ -> 
+	    raise Argument_error
+    ) with Argument_error -> 
+      eprintf "wrong number of arguments\n";
+      eprintf "usage: sksdump numkeys dumpdir [dumpname]\n";
+      flush stderr;
+      exit (-1)
 end
